@@ -31,8 +31,8 @@ EOS
    exit(-1)
 end
 
-if defined?(__CYGWIN__) && VERSION < '1.7'
-   have_library("narray") || raise("ERROR: narray brary is not found")
+if /cygwin|mingw/ =~ RUBY_PLATFORM
+   have_library("narray") || raise("ERROR: narray library is not found")
 end
 
 create_makefile "numru/netcdfraw"
