@@ -99,7 +99,7 @@ module NumRu
 	      break
 	   end
 	 rescue
-	   raise "cannot generate tempfile `%s'" % tmpname if n >= Max_Try
+	   raise NetcdfError, "cannot generate tempfile `%s'" % tmpname if n >= Max_Try
 	 end
 	 n += 1
        end
@@ -115,7 +115,7 @@ module NumRu
        # Same as def_var but defines dimensions first if needed.
        # Use zero in shape to define an unlimited dimension.
        if (shape_ul0.length != dimnames.length ) then
-	  raise 'lengths of shape and dimnames do not agree'
+	  raise ArgumentError, 'lengths of shape and dimnames do not agree'
        end
        dims = []
        dimnames.each_index{ |i|
@@ -123,7 +123,7 @@ module NumRu
 	  if ( dim != nil ) then
 	     # dim exists --> check the length
 	     if (shape_ul0[i] != dim.length_ul0 ) then
-		raise "dimension length do not agree: #{i}th dim: "+\
+		raise ArgumentError, "dimension length do not agree: #{i}th dim: "+\
 		"#{shape_ul0[i]} and #{dim.length_ul0}"
 	     end
 	     dims.push(dim)
@@ -164,8 +164,9 @@ module NumRu
        if names == nil
 	  dims = (0..ndims()-1).collect{|dimid| id2dim(dimid)}
        else
-	  raise "names is not an array" if ! names.is_a?(Array)
+	  raise TypeError, "names is not an array" if ! names.is_a?(Array)
 	  dims = names.collect{|name| dim(name)}
+	  raise ArgumentError, "One or more dimensions do not exist" if dims.include?(nil)
        end
        dims
     end
@@ -174,8 +175,9 @@ module NumRu
        if names == nil
 	  vars = (0..nvars()-1).collect{ |varid| id2var(varid) }
        else
-	  raise "names is not an array" if ! names.is_a?(Array)
+	  raise TypeError, "names is not an array" if ! names.is_a?(Array)
 	  vars = names.collect{|name| var(name)}
+	  raise ArgumentError, "One or more variables do not exist" if vars.include?(nil)
        end
        vars
     end
@@ -224,7 +226,7 @@ module NumRu
 	  if(file.is_a?(String))
 	     file = NetCDF.open(file,mode,share)
 	  elsif(!file.is_a?(NetCDF))
-	    raise "1st arg must be a NetCDF (file object) or a String (path)"
+	    raise TypeError, "1st arg must be a NetCDF (file object) or a String (path)"
 	  end
 	  file.var(varname)
        end
@@ -296,9 +298,9 @@ module NumRu
 	    if csf.is_a?(NArray) then  # --> should be a numeric
 	       csf = csf[0]
 	    elsif csf.is_a?(String)
-	       raise "scale_factor is not a numeric"
+	       raise TypeError, "scale_factor is not a numeric"
 	    end
-	    if(csf == 0) then; raise "zero scale_factor"; end
+	    if(csf == 0) then; raise NetcdfError, "zero scale_factor"; end
 	 else
 	    csf = 1.0      # assume 1 if not defined
 	 end
@@ -307,7 +309,7 @@ module NumRu
 	    if cao.is_a?(NArray) then  # --> should be a numeric
 	       cao = cao[0]
 	    elsif csf.is_a?(String)
-	       raise "add_offset is not a numeric"
+	       raise NetcdfError, "add_offset is not a numeric"
 	    end
 	 else
 	    cao = 0.0      # assume 0 if not defined
@@ -316,6 +318,21 @@ module NumRu
       end
    end
 
+   @@scaled_get_type = nil
+   class << NetCDFVar
+     def scaled_get_type
+       @@scaled_get_type
+     end
+     def scaled_get_type=(na_type)
+       if [NArray::BYTE, NArray::SINT, NArray::INT, 
+	   NArray::SFLOAT, NArray::FLOAT].include?(na_type)
+	 @@scaled_get_type = na_type
+       else
+	 raise ArgumentError, "Arg must be one of NArray::BYTE, NArray::SINT, NArray::INT, NArray::SFLOAT, NArray::FLOAT"
+       end
+     end
+   end
+
    def scaled_get(hash=nil)
       sf = att('scale_factor')
       ao = att('add_offset')
@@ -328,9 +345,9 @@ module NumRu
 	    if csf.is_a?(NArray) then  # --> should be a numeric
 	       csf = csf[0]
 	    elsif csf.is_a?(String)
-	       raise "scale_factor is not a numeric"
+	       raise NetcdfError, "scale_factor is not a numeric"
 	    end
-	    if(csf == 0) then; raise "zero scale_factor"; end
+	    if(csf == 0) then; raise NetcdfError, "zero scale_factor"; end
 	 else
 	    csf = 1.0      # assume 1 if not defined
 	 end
@@ -339,20 +356,26 @@ module NumRu
 	    if cao.is_a?(NArray) then  # --> should be a numeric
 	       cao = cao[0]
 	    elsif csf.is_a?(String)
-	       raise "add_offset is not a numeric"
+	       raise NetcdfError, "add_offset is not a numeric"
 	    end
 	 else
 	    cao = 0.0      # assume 0 if not defined
 	 end
 	 var = simple_get(hash)
 	 case (vartype)
-	 when "int"
-	    var = var.to_type( NArray::FLOAT )
+	 when "int","float","sfloat"
+	    tp = @@scaled_get_type || NArray::FLOAT
 	 when "byte","sint","char"
-	    var = var.to_type( NArray::SFLOAT )
+	    tp = @@scaled_get_type || NArray::SFLOAT
 	 else
+	   raise "Sorry, must be a bag"
+	 end
+	 if tp <= NArray::INT && Float===csf
+	   ( var.to_type(NArray::FLOAT) * csf + cao ).to_type( tp )
+	          #^because NArray's type is not upgraded upon op with Float
+	 else  
+	   var.to_type(tp) * csf + cao
 	 end
-	 csf*var+cao
       end
    end
 
@@ -456,7 +479,7 @@ module NumRu
 	 end
        end
      else
-       raise NetcdfError,"{'start'}=>[ARRAY] or {'index'}=>[ARRAY] is needed"
+       raise ArgumentError,"{'start'}=>[ARRAY] or {'index'}=>[ARRAY] is needed"
      end
    end
 
@@ -515,7 +538,7 @@ module NumRu
 	 elsif t_var == "float"
 	   get_vars_float(h_sta,nil,nil)
 	 else 
-	   raise NetcdfError "varialbe type #{t_var} isn't supported in netCDF"
+	   raise NetcdfError, "varialbe type #{t_var} isn't supported in netCDF"
 	 end
        elsif endq == true && strq == false
 	 h_end = hash["end"]
@@ -571,13 +594,32 @@ module NumRu
 	 end
        end
      else
-       raise NetcdfError,"{'start'}=>{ARRAY} or {'index'}=>{ARRAY} is needed"
+       raise ArgumentError,"{'start'}=>{ARRAY} or {'index'}=>{ARRAY} is needed"
      end
    end
 
    alias get simple_get
 
+   def __rubber_expansion( args )
+     if (id = args.index(false))  # substitution into id
+       # false is incuded
+       alen = args.length
+       if args.rindex(false) != id
+	 raise ArguemntError,"only one rubber dimension is permitted"
+       elsif alen > rank+1
+	 raise ArgumentError, "too many args"
+       end
+       ar = ( id!=0 ? args[0..id-1] : [] )
+       args = ar + [true]*(rank-alen+1) + args[id+1..-1]
+     elsif args.length == 0   # to support empty [], []=
+       args = [true]*rank
+     end
+     args
+   end
+   private :__rubber_expansion
+
    def [](*a)
+     a = __rubber_expansion(a)
      first = Array.new
      last = Array.new
      stride = Array.new
@@ -595,7 +637,7 @@ module NumRu
 	 r = (i.to_a[0])[0]
          s = (i.to_a[0])[1]
          if ( !( r.is_a?(Range) ) || ! ( s.is_a?(Integer) ) )
-	    raise "Hash argument must be {a_Range, step}"
+	    raise TypeError, "Hash argument must be {a_Range, step}"
 	 end
 	 first.push(r.first) 
 	 last.push(r.exclude_end? ? r.last-1 : r.last)
@@ -605,6 +647,27 @@ module NumRu
 	 first.push(0)
 	 last.push(-1)
 	 stride.push(1)
+       elsif( i.is_a?(Array) || i.is_a?(NArray))
+	 a_new = a.dup
+	 at = a.index(i)
+	 i = NArray.to_na(i) if i.is_a?(Array)
+	 for n in 0..i.length-1
+	   a_new[at] = i[n]..i[n]
+	   na_tmp = self[*a_new]
+	   if n==0 then
+	     k = at
+	     a[0..at-1].each{|x| if x.is_a?(Fixnum) then k -= 1 end}
+	     shape_tmp = na_tmp.shape
+	     shape_tmp[k] = i.length
+	     na = NArray.new(na_tmp.typecode,*shape_tmp)
+	     index_tmp = Array.new(shape_tmp.length,true)
+	   end
+	   index_tmp[k] = n..n
+	   na[*index_tmp] = na_tmp
+	 end
+	 return na
+       else
+	 raise TypeError, "argument must be Fixnum, Range, Hash, TrueClass, Array, or NArray"
        end
      }
 
@@ -623,6 +686,7 @@ module NumRu
 
    def []=(*a)
      val = a.pop
+     a = __rubber_expansion(a)
      first = Array.new
      last = Array.new
      stride = Array.new
@@ -640,7 +704,7 @@ module NumRu
 	 r = (i.to_a[0])[0]
          s = (i.to_a[0])[1]
          if ( !( r.is_a?(Range) ) || ! ( s.is_a?(Integer) ) )
-	    raise "Hash argument must be {first..last, step}"
+	    raise ArgumentError, "Hash argument must be {first..last, step}"
 	 end
 	 first.push(r.first) 
 	 last.push(r.exclude_end? ? r.last-1 : r.last)
@@ -650,6 +714,35 @@ module NumRu
 	 first.push(0)
 	 last.push(-1)
 	 stride.push(1)
+       elsif(i.is_a?(Array) || i.is_a?(NArray))
+	 a_new = a.dup
+	 at = a.index(i)
+	 i = NArray.to_na(i) if i.is_a?(Array)
+	 val = NArray.to_na(val) if val.is_a?(Array)
+	 rank_of_subset = a.dup.delete_if{|v| v.is_a?(Fixnum)}.length
+	 if val.rank != rank_of_subset
+	   raise "rank of the rhs (#{val.rank}) is not equal to the rank "+
+                 "of the subset specified by #{a.inspect} (#{rank_of_subset})"
+	 end
+	 k = at
+	 a[0..at-1].each{|x| if x.is_a?(Fixnum) then k -= 1 end}
+	 if i.length != val.shape[k]
+	   raise "length of the #{k+1}-th dim of rhs is incorrect "+
+                 "(#{i.length} for #{val.shape[k]})"
+	 end
+	 index_tmp = Array.new(val.rank,true) if val.is_a?(NArray)
+	 for n in 0..i.length-1
+	   a_new[at] = i[n]..i[n]
+	   if val.is_a?(NArray) then
+	     index_tmp[k] = n..n
+	     self[*a_new] = val[*index_tmp]
+	   else
+	     self[*a_new] = val
+	   end
+	 end
+	 return self
+       else
+	 raise TypeError, "argument must be Fixnum, Range, Hash, TrueClass, Array, or NArray"
        end
      }
 
